GET_NT_VERSION		equ	3306h
NT_MAJOR_VERSION	equ	05
NT_MINOR_VERSION	equ	50

CR			equ	0Dh
LF			equ	0Ah

IDLE_ID			equ	0C1h

;**-------------- Status Values ----------------**
STAT_OK      equ  0000h     ;ok
STAT_DONE    equ  0100h     ;function complete
STAT_GF      equ  800Ch     ;general failure

			struc	IRP     ;common to all commands
			resb    1	;length of request header
			resb	1	;unit code of device
.CMD			resb	1	;command code
.STA			resw	1	;completion status, 16-bits
			resq	1	;reserved for DOS
			resb	1	;this field varies with command
.OFF			resw	1	;offset of data
.SEG			resw	1	;segment of data
.CNT			resw	1	;byte count (length) of data
			endstruc	;end of common portion


%macro			DrvHd	3
			dd	-1
			dw	08000h
			dw	%1
			dw	%2
			db      %3
%endmacro
                        %include 'isvbop.inc'
			bits	16
			org	0
			align	16
drive_header:
			DrvHd	DrvStart, DrvIntr , 'IDLE    '
IRP_P:
IRP_O			dw	0
IRP_S			dw	0

;**------------- Define Stack Space ------------**
STK_P:
STK_O			dw	0	;Save original stack pointer
STK_S	  		dw	0	;Save original stack segment
STACK	times	200	db	0
TOP_STK:

DrvStart:
			mov	[cs:IRP_O],bx     ;save offset address
			mov	[cs:IRP_S],es     ;save segment address
			retf
DrvIntr:
			push	ds        ;save all modified registers
			push	es        ;DOS has stack for 20 pushes
			push	ax
			push	bx
			push	cx
			push	dx
			push	di
			push	si
			push	bp

			mov	[cs:STK_O],sp   ;save original stack ptr
			mov	[cs:STK_S],ss   ;save original stack seg
			cli                  ;disable for stack ops
			mov	ax,cs           ;setup new stack ptr
			mov	ss,ax           ;setup new stack seg
			mov	sp,TOP_STK
			sti                  ;restore flags back
			cld                  ;all moves are forward

			les	bx,[cs:IRP_P]  ;load req hdr adr in es:bx
			mov	al,[es:bx+IRP.CMD]
			cmp	al,0           ;check for init command
			jne	.exit         ;command 0 = init
			call	Init_Fence
.exit:
			les	bx,[cs:IRP_P]  ;restore ES:BX
			or	di,STAT_DONE   ;add "DONE" bit to status
			mov	[es:bx+IRP.STA],di  ;save status in requ hdr

			cli                 ;disable ints for stack op
			lss	sp, [cs:STK_P] ;restore stack
			sti                 ;re-enable interrupts

			pop  bp             ;restore registers
			pop  si
			pop  di
			pop  dx
			pop  cx
			pop  bx
			pop  ax
			pop  es
			pop  ds
			retf

Int28Hook:
                        push	word 28h
			push    ax                          ;put ax on stack
			mov     ax, [cs:hVDD]
			DispatchCall
			add     sp, 4                       ;vdd has set ax accordingly

			iret
;-----------------------------------------------------------------------------

Int2FHook:
			cmp     ax, 1680h                   ; AppIdle
			je      callvdd
			cmp     ah, IDLE_ID                 ;IDLE?
			jne     int2fchain                  ;no

			cmp     al, 0Bh                     ;command too high?
			je      try_0b
			ja      int2fchain                  ;yes
callvdd:
                        push	word 2Fh
			push    ax                          ;put ax on stack
			mov     ax, [cs:hVDD]
			DispatchCall
			add     sp, 4                       ;vdd has set ax accordingly
			iret                                ;svc handled, return to caller

try_0b:
			xor	ax, ax
			mov	bx,0ABCDh
int2f_done:
			iret

int2fchain:
			jmp     far [cs:oldint]

;-----------------------------------------------------------
RegisterVDD:
			mov	ax, cs
			mov	ds, ax
			mov	es, ax

			; Load vcdex.dll
			lea	si, [DllName]	; ds:si = dll name
			lea	di, [InitFunc]	; es:di = init routine
			lea	bx, [DispFunc]	; ds:bx = dispatch routine

			RegisterModule
			jc      errorexit	; jif error
			mov     [cs:hVDD],ax	; save handle
errorexit:
			ret

;*-----------------------  TSR Data Area ---------------------*
oldint			dd      0
oldint28		dd      0
hVDD			dw      0

fVDDChecked		db	0       ; 0 - VDD never called. 1 - VDD once called.

DllName			db	'IDLE.DLL',0
InitFunc		db	'VDDRegisterInit',0
DispFunc		db	'VDDDispatch',0

			align	16
Init_Fence:
;*-------------------------- Initialization Code ----------------------*
idlent:
        ; first check that we are running under NT

			mov     ax, GET_NT_VERSION
			int     21h
			cmp     bl, NT_MAJOR_VERSION
			je      idle_chk_more
			jmp     idle_badver
idle_chk_more:
			cmp     bh, NT_MINOR_VERSION
			je      idle_ver_ok
			jmp     idle_badver

idle_ver_ok:
			; Now check that this TSR is'nt already installed
			mov     ah,IDLE_ID
			mov     al,0Bh                  ; call function 0b
			int     2Fh			; int 2f

			cmp     bx,0ABCDh
			jne     idle_chks_done
			jmp     idle_installed

idle_chks_done:
			; hook int 2f
			mov     ah, 35h			; DOS_GET_VECTOR
			mov     al, 2Fh			; 2f
			int     21h			; get old vector
			mov     [cs:oldint],bx		; save old vector here
			mov     [cs:oldint+2],es

			lea	dx, [Int2FHook]
			push    cs			; get current code segment
			pop     ds
			mov     ah, 25h			; DOS_SET_VECTOR
			mov     al, 2Fh			; vector to hook
			int     21h			; hook that vector

			; hook int 28
			mov     ah, 35h			; DOS_GET_VECTOR
			mov     al, 28h			; 28
			int     21h			; get old vector
			mov     [cs:oldint28],bx		; save old vector here
			mov     [cs:oldint28+2],es

			lea	dx, [Int28Hook]
			push    cs			; get current code segment
			pop     ds
			mov     ah, 25h			; DOS_SET_VECTOR
			mov     al, 28h			; vector to hook
			int     21h			; hook that vector

			call	RegisterVDD
			mov	di, STAT_OK		; load OK status
			jnc	idle_exit		;
			mov	di, STAT_GF		; failure

			; restore interrupts
			lds	dx, [cs:oldint]
			mov     ah, 25h			; DOS_SET_VECTOR
			mov     al, 2Fh			; vector to hook
			int     21h			; hook that vector

			lds	dx, [cs:oldint28]
			mov     ah, 25h			; DOS_SET_VECTOR
			mov     al, 28h			; vector to hook
			int     21h			; hook that vector

			push    cs			; get current code segment
			pop     ds

			jmp     short idle_exit

idle_badver:
			lea	dx, [Message1]
			push    cs
			pop     ds
			mov     ah,09h
			int     21h
			mov	di, STAT_GF		; failure
			jmp     short idle_exit

idle_installed:
			lea	dx, [Message2]
			push    cs
			pop     ds
			mov     ah,09h
			int     21h
			mov	di, STAT_GF		; failure

idle_exit:
			lea	ax, [Init_Fence]
			les	bx,[cs:IRP_P]  ;load req hdr adr in es:bx
			mov	[es:bx+IRP.OFF], ax
			mov	[es:bx+IRP.SEG], cs
			ret

Message1		db  CR,LF,'IDLE Will Run Only Under Windows NT.',CR,LF,'$'
Message2		db  CR,LF,'IDLE is already installed.',CR,LF,'$'
